@@ -33,7 +33,6 @@
 #include "nautilus-icon-names.h"
 #include "nautilus-trash-monitor.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-gtk4-helpers.h"
 
 #include "nautilus-window-slot-dnd.h"
 
@@ -78,8 +77,6 @@ typedef struct
 
     NautilusPathBar *path_bar;
 
-    GtkGesture *multi_press_gesture;
-
     guint ignore_changes : 1;
     guint is_root : 1;
 } ButtonData;
@@ -95,25 +92,23 @@ struct _NautilusPathBar
     gpointer current_button_data;
 
     GList *button_list;
-    gulong settings_signal_id;
 
     GActionGroup *action_group;
 
     NautilusFile *context_menu_file;
-    GtkPopover *current_view_menu_popover;
+    GtkPopoverMenu *current_view_menu_popover;
     GtkWidget *current_view_menu_button;
-    GtkPopover *button_menu_popover;
+    GtkWidget *button_menu_popover;
     GMenu *current_view_menu;
     GMenu *extensions_section;
     GMenu *templates_submenu;
     GMenu *button_menu;
+
+    gchar *os_name;
 };
 
 G_DEFINE_TYPE (NautilusPathBar, nautilus_path_bar, GTK_TYPE_BOX);
 
-static void nautilus_path_bar_check_icon_theme (NautilusPathBar *self);
-static void nautilus_path_bar_update_button_appearance (ButtonData *button_data,
-                                                        gboolean    current_dir);
 static void nautilus_path_bar_update_button_state (ButtonData *button_data,
                                                    gboolean    current_dir);
 static void nautilus_path_bar_update_path (NautilusPathBar *self,
@@ -211,22 +206,33 @@ action_pathbar_properties (GSimpleAction *action,
 }
 
 static void
-on_adjustment_changed (GtkAdjustment *adjustment)
+on_adjustment_changed (GtkAdjustment   *adjustment,
+                       NautilusPathBar *self)
 {
-    gboolean rtl = (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL);
+    /* Automatically scroll to the end, to reveal the current folder. */
+    g_autoptr (AdwAnimation) anim = NULL;
+    anim = adw_timed_animation_new (GTK_WIDGET (self),
+                                    gtk_adjustment_get_value (adjustment),
+                                    gtk_adjustment_get_upper (adjustment),
+                                    800,
+                                    adw_property_animation_target_new (G_OBJECT (adjustment), "value"));
+    adw_timed_animation_set_easing (ADW_TIMED_ANIMATION (anim), ADW_EASE_OUT_CUBIC);
+    adw_animation_play (anim);
+}
 
-    /* Automatically scroll to the end, to keep the current folder visible */
-    gtk_adjustment_set_value (adjustment, (rtl ?
-                                           gtk_adjustment_get_lower (adjustment) :
-                                           gtk_adjustment_get_upper (adjustment)));
+static void
+on_page_size_changed (GtkAdjustment *adjustment)
+{
+    /* When window is resized, immediately set new value, otherwise we would get
+     * an underflow gradient for an moment. */
+    gtk_adjustment_set_value (adjustment, gtk_adjustment_get_upper (adjustment));
 }
 
 static gboolean
 bind_current_view_menu_model_to_popover (NautilusPathBar *self)
 {
-    gtk_popover_bind_model (self->current_view_menu_popover,
-                            G_MENU_MODEL (self->current_view_menu),
-                            NULL);
+    gtk_popover_menu_set_menu_model (self->current_view_menu_popover,
+                                     G_MENU_MODEL (self->current_view_menu));
     return G_SOURCE_REMOVE;
 }
 
@@ -237,7 +243,9 @@ nautilus_path_bar_init (NautilusPathBar *self)
     GtkBuilder *builder;
     g_autoptr (GError) error = NULL;
 
-    self->scrolled = gtk_scrolled_window_new (NULL, NULL);
+    self->os_name = g_get_os_info (G_OS_INFO_KEY_NAME);
+
+    self->scrolled = gtk_scrolled_window_new ();
     /* Scroll horizontally only and don't use internal scrollbar. */
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->scrolled),
                                     /* hscrollbar-policy */ GTK_POLICY_EXTERNAL,
@@ -246,17 +254,20 @@ nautilus_path_bar_init (NautilusPathBar *self)
     gtk_box_append (GTK_BOX (self), self->scrolled);
 
     adjustment = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self->scrolled));
-    g_signal_connect (adjustment, "changed", (GCallback) on_adjustment_changed, NULL);
+    g_signal_connect (adjustment, "changed", G_CALLBACK (on_adjustment_changed), self);
+    g_signal_connect (adjustment, "notify::page-size", G_CALLBACK (on_page_size_changed), self);
 
     self->buttons_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (self->scrolled), self->buttons_box);
 
     self->current_view_menu_button = gtk_menu_button_new ();
+    gtk_widget_add_css_class (self->current_view_menu_button, "flat");
     gtk_menu_button_set_child (GTK_MENU_BUTTON (self->current_view_menu_button),
-                               gtk_image_new_from_icon_name ("view-more-symbolic",
-                                                             GTK_ICON_SIZE_MENU));
+                               gtk_image_new_from_icon_name ("view-more-symbolic"));
     gtk_box_append (GTK_BOX (self), self->current_view_menu_button);
 
+    gtk_widget_set_tooltip_text (self->current_view_menu_button, _("Current Folder Menu"));
+
     builder = gtk_builder_new ();
 
     /* Add context menu for pathbar buttons */
@@ -268,21 +279,16 @@ nautilus_path_bar_init (NautilusPathBar *self)
         g_error ("Failed to add pathbar-context-menu.ui: %s", error->message);
     }
     self->button_menu = g_object_ref_sink (G_MENU (gtk_builder_get_object (builder, "button-menu")));
-    self->button_menu_popover = g_object_ref_sink (GTK_POPOVER (gtk_popover_new_from_model (NULL,
-                                                                                            G_MENU_MODEL (self->button_menu))));
+    self->button_menu_popover = gtk_popover_menu_new_from_model (G_MENU_MODEL (self->button_menu));
+    gtk_widget_set_parent (self->button_menu_popover, GTK_WIDGET (self));
+    gtk_popover_set_has_arrow (GTK_POPOVER (self->button_menu_popover), FALSE);
+    gtk_widget_set_halign (self->button_menu_popover, GTK_ALIGN_START);
 
-    /* Add current location menu, which matches the view's background context menu */
-    gtk_builder_add_from_resource (builder,
-                                   "/org/gnome/nautilus/ui/nautilus-files-view-context-menus.ui",
-                                   &error);
-    if (error != NULL)
-    {
-        g_error ("failed to add files-view-context-menus.ui: %s", error->message);
-    }
-    self->current_view_menu = g_object_ref_sink (G_MENU (gtk_builder_get_object (builder, "background-menu")));
+    /* Add current location menu, which shares features with the view's background context menu */
+    self->current_view_menu = g_object_ref_sink (G_MENU (gtk_builder_get_object (builder, "current-view-menu")));
     self->extensions_section = g_object_ref (G_MENU (gtk_builder_get_object (builder, "background-extensions-section")));
     self->templates_submenu = g_object_ref (G_MENU (gtk_builder_get_object (builder, "templates-submenu")));
-    self->current_view_menu_popover = g_object_ref_sink (GTK_POPOVER (gtk_popover_new (NULL)));
+    self->current_view_menu_popover = g_object_ref_sink (GTK_POPOVER_MENU (gtk_popover_menu_new_from_model (NULL)));
 
     g_object_unref (builder);
 
@@ -290,12 +296,8 @@ nautilus_path_bar_init (NautilusPathBar *self)
                                  GTK_WIDGET (self->current_view_menu_popover));
     bind_current_view_menu_model_to_popover (self);
 
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)),
-                                 "linked");
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)),
-                                 "nautilus-path-bar");
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self->buttons_box)),
-                                 "path-buttons-box");
+    gtk_widget_set_name (GTK_WIDGET (self), "NautilusPathBar");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "linked");
 
     /* Action group */
     self->action_group = G_ACTION_GROUP (g_simple_action_group_new ());
@@ -319,34 +321,19 @@ nautilus_path_bar_finalize (GObject *object)
     g_clear_object (&self->extensions_section);
     g_clear_object (&self->templates_submenu);
     g_clear_object (&self->button_menu);
-    g_clear_object (&self->button_menu_popover);
     g_clear_object (&self->current_view_menu_popover);
+    g_free (self->os_name);
 
     unschedule_pop_up_context_menu (NAUTILUS_PATH_BAR (object));
 
     G_OBJECT_CLASS (nautilus_path_bar_parent_class)->finalize (object);
 }
 
-/* Removes the settings signal handler.  It's safe to call multiple times */
-static void
-remove_settings_signal (NautilusPathBar *self,
-                        GdkScreen       *screen)
-{
-    GtkSettings *settings;
-
-    settings = gtk_settings_get_for_screen (screen);
-
-    g_clear_signal_handler (&self->settings_signal_id, settings);
-}
-
 static void
 nautilus_path_bar_dispose (GObject *object)
 {
-    NautilusPathBar *self;
+    NautilusPathBar *self = NAUTILUS_PATH_BAR (object);
 
-    self = NAUTILUS_PATH_BAR (object);
-
-    remove_settings_signal (self, gtk_widget_get_screen (GTK_WIDGET (object)));
     nautilus_path_bar_clear_buttons (self);
 
     G_OBJECT_CLASS (nautilus_path_bar_parent_class)->dispose (object);
@@ -359,9 +346,14 @@ get_dir_name (ButtonData *button_data)
     {
         case ROOT_BUTTON:
         {
+            if (button_data->path_bar != NULL &&
+                button_data->path_bar->os_name != NULL)
+            {
+                return button_data->path_bar->os_name;
+            }
             /* Translators: This is the label used in the pathbar when seeing
              * the root directory (also known as /) */
-            return _("Computer");
+            return _("Operating System");
         }
 
         case ADMIN_ROOT_BUTTON:
@@ -393,30 +385,6 @@ get_dir_name (ButtonData *button_data)
     }
 }
 
-static void
-nautilus_path_bar_style_updated (GtkWidget *widget)
-{
-    GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->style_updated (widget);
-
-    nautilus_path_bar_check_icon_theme (NAUTILUS_PATH_BAR (widget));
-}
-
-static void
-nautilus_path_bar_screen_changed (GtkWidget *widget,
-                                  GdkScreen *previous_screen)
-{
-    if (GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->screen_changed)
-    {
-        GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->screen_changed (widget, previous_screen);
-    }
-    /* We might nave a new settings, so we remove the old one */
-    if (previous_screen)
-    {
-        remove_settings_signal (NAUTILUS_PATH_BAR (widget), previous_screen);
-    }
-    nautilus_path_bar_check_icon_theme (NAUTILUS_PATH_BAR (widget));
-}
-
 static void
 button_data_free (ButtonData *button_data)
 {
@@ -430,8 +398,6 @@ button_data_free (ButtonData *button_data)
         nautilus_file_unref (button_data->file);
     }
 
-    g_clear_object (&button_data->multi_press_gesture);
-
     g_free (button_data);
 }
 
@@ -439,17 +405,12 @@ static void
 nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
 {
     GObjectClass *gobject_class;
-    GtkWidgetClass *widget_class;
 
     gobject_class = (GObjectClass *) path_bar_class;
-    widget_class = (GtkWidgetClass *) path_bar_class;
 
     gobject_class->finalize = nautilus_path_bar_finalize;
     gobject_class->dispose = nautilus_path_bar_dispose;
 
-    widget_class->style_updated = nautilus_path_bar_style_updated;
-    widget_class->screen_changed = nautilus_path_bar_screen_changed;
-
     path_bar_signals [OPEN_LOCATION] =
         g_signal_new ("open-location",
                       G_OBJECT_CLASS_TYPE (path_bar_class),
@@ -474,6 +435,8 @@ void
 nautilus_path_bar_set_templates_menu (NautilusPathBar *self,
                                       GMenuModel      *menu)
 {
+    gint i;
+
     g_return_if_fail (NAUTILUS_IS_PATH_BAR (self));
 
     if (!gtk_widget_is_visible (GTK_WIDGET (self->current_view_menu_popover)))
@@ -482,62 +445,18 @@ nautilus_path_bar_set_templates_menu (NautilusPathBar *self,
          * templates menu is set. Unbinding the model is the only way to clear
          * all children. After that's done, on idle, we rebind it.
          * See https://gitlab.gnome.org/GNOME/nautilus/-/issues/1705 */
-        gtk_popover_bind_model (self->current_view_menu_popover, NULL, NULL);
+        gtk_popover_menu_set_menu_model (self->current_view_menu_popover, NULL);
     }
 
     nautilus_gmenu_set_from_model (self->templates_submenu, menu);
     g_idle_add ((GSourceFunc) bind_current_view_menu_model_to_popover, self);
-}
 
-/* Changes the icons wherever it is needed */
-static void
-reload_icons (NautilusPathBar *self)
-{
-    GList *list;
-
-    for (list = self->button_list; list; list = list->next)
-    {
-        ButtonData *button_data;
-
-        button_data = BUTTON_DATA (list->data);
-        if (button_data->type != NORMAL_BUTTON || button_data->is_root)
-        {
-            nautilus_path_bar_update_button_appearance (button_data,
-                                                        list->next == NULL);
-        }
-    }
-}
-
-/* Callback used when a GtkSettings value changes */
-static void
-settings_notify_cb (GObject         *object,
-                    GParamSpec      *pspec,
-                    NautilusPathBar *self)
-{
-    const char *name;
-
-    name = g_param_spec_get_name (pspec);
-
-    if (!strcmp (name, "gtk-icon-theme-name") || !strcmp (name, "gtk-icon-sizes"))
-    {
-        reload_icons (self);
-    }
-}
-
-static void
-nautilus_path_bar_check_icon_theme (NautilusPathBar *self)
-{
-    GtkSettings *settings;
-
-    if (self->settings_signal_id)
-    {
-        return;
-    }
-
-    settings = gtk_settings_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (self)));
-    self->settings_signal_id = g_signal_connect (settings, "notify", G_CALLBACK (settings_notify_cb), self);
-
-    reload_icons (self);
+    i = nautilus_g_menu_model_find_by_string (G_MENU_MODEL (self->current_view_menu),
+                                              "nautilus-menu-item",
+                                              "templates-submenu");
+    nautilus_g_menu_replace_string_in_item (self->current_view_menu, i,
+                                            "hidden-when",
+                                            (menu == NULL) ? "action-missing" : NULL);
 }
 
 /* Public functions and their helpers */
@@ -562,7 +481,7 @@ nautilus_path_bar_show_current_location_menu (NautilusPathBar *self)
 {
     g_return_if_fail (NAUTILUS_IS_PATH_BAR (self));
 
-    g_signal_emit_by_name (self->current_view_menu_button, "activate");
+    gtk_menu_button_popup (GTK_MENU_BUTTON (self->current_view_menu_button));
 }
 
 static void
@@ -582,7 +501,7 @@ button_clicked_cb (GtkButton *button,
 
     if (g_file_equal (button_data->path, self->current_path))
     {
-        nautilus_path_bar_show_current_location_menu (self);
+        return;
     }
     else
     {
@@ -595,7 +514,7 @@ button_clicked_cb (GtkButton *button,
 static void
 real_pop_up_pathbar_context_menu (NautilusPathBar *self)
 {
-    gtk_popover_popup (self->button_menu_popover);
+    gtk_popover_popup (GTK_POPOVER (self->button_menu_popover));
 }
 
 static void
@@ -668,16 +587,17 @@ pop_up_pathbar_context_menu (NautilusPathBar *self,
 
 
 static void
-on_multi_press_gesture_pressed (GtkGestureMultiPress *gesture,
-                                gint                  n_press,
-                                gdouble               x,
-                                gdouble               y,
-                                gpointer              user_data)
+on_click_gesture_pressed (GtkGestureClick *gesture,
+                          gint             n_press,
+                          gdouble          x,
+                          gdouble          y,
+                          gpointer         user_data)
 {
     ButtonData *button_data;
     NautilusPathBar *self;
     guint current_button;
     GdkModifierType state;
+    double x_in_pathbar, y_in_pathbar;
 
     if (n_press != 1)
     {
@@ -687,7 +607,12 @@ on_multi_press_gesture_pressed (GtkGestureMultiPress *gesture,
     button_data = BUTTON_DATA (user_data);
     self = button_data->path_bar;
     current_button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
-    gtk_get_current_event_state (&state);
+    state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (gesture));
+
+    gtk_widget_translate_coordinates (GTK_WIDGET (button_data->button),
+                                      GTK_WIDGET (self),
+                                      x, y,
+                                      &x_in_pathbar, &y_in_pathbar);
 
     switch (current_button)
     {
@@ -706,12 +631,12 @@ on_multi_press_gesture_pressed (GtkGestureMultiPress *gesture,
         {
             if (g_file_equal (button_data->path, self->current_path))
             {
-                nautilus_path_bar_show_current_location_menu (self);
+                break;
             }
             else
             {
-                gtk_popover_set_relative_to (self->button_menu_popover,
-                                             button_data->button);
+                gtk_popover_set_pointing_to (GTK_POPOVER (self->button_menu_popover),
+                                             &(GdkRectangle){x_in_pathbar, y_in_pathbar, 0, 0});
                 pop_up_pathbar_context_menu (self, button_data->file);
             }
         }
@@ -830,6 +755,8 @@ nautilus_path_bar_update_button_appearance (ButtonData *button_data,
     {
         gtk_label_set_text (GTK_LABEL (button_data->label), dir_name);
 
+        gtk_widget_set_tooltip_text (button_data->button, dir_name);
+
         if (current_dir)
         {
             /* We want to avoid ellipsizing the current directory name, but
@@ -863,7 +790,7 @@ nautilus_path_bar_update_button_appearance (ButtonData *button_data,
     icon = get_gicon (button_data);
     if (icon != NULL)
     {
-        gtk_image_set_from_gicon (GTK_IMAGE (button_data->image), icon, GTK_ICON_SIZE_MENU);
+        gtk_image_set_from_gicon (GTK_IMAGE (button_data->image), icon);
         gtk_widget_show (GTK_WIDGET (button_data->image));
         g_object_unref (icon);
     }
@@ -1084,6 +1011,7 @@ make_button_data (NautilusPathBar *self,
 {
     GFile *path;
     GtkWidget *child = NULL;
+    GtkEventController *controller;
     ButtonData *button_data;
 
     path = nautilus_file_get_location (file);
@@ -1094,6 +1022,7 @@ make_button_data (NautilusPathBar *self,
     setup_button_type (button_data, self, path);
     button_data->button = gtk_button_new ();
     gtk_widget_set_focus_on_click (button_data->button, FALSE);
+    gtk_widget_set_name (button_data->button, "NautilusPathButton");
 
     /* TODO update button type when xdg directories change */
 
@@ -1185,23 +1114,22 @@ make_button_data (NautilusPathBar *self,
     }
 
     gtk_button_set_child (GTK_BUTTON (button_data->button), child);
-    gtk_widget_show_all (button_data->container);
-
-    nautilus_path_bar_update_button_state (button_data, current_dir);
+    gtk_widget_show (button_data->container);
 
     button_data->path_bar = self;
 
+    nautilus_path_bar_update_button_state (button_data, current_dir);
+
     g_signal_connect (button_data->button, "clicked", G_CALLBACK (button_clicked_cb), button_data);
 
     /* A gesture is needed here, because GtkButton doesn’t react to middle- or
      * secondary-clicking.
      */
-    button_data->multi_press_gesture = gtk_gesture_multi_press_new (button_data->button);
-
-    gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (button_data->multi_press_gesture), 0);
-
-    g_signal_connect (button_data->multi_press_gesture, "pressed",
-                      G_CALLBACK (on_multi_press_gesture_pressed), button_data);
+    controller = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
+    gtk_widget_add_controller (button_data->button, controller);
+    gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (controller), 0);
+    g_signal_connect (controller, "pressed",
+                      G_CALLBACK (on_click_gesture_pressed), button_data);
 
     nautilus_drag_slot_proxy_init (button_data->button, button_data->file, NULL);
 
@@ -1254,16 +1182,16 @@ nautilus_path_bar_update_path (NautilusPathBar *self,
 
     nautilus_path_bar_clear_buttons (self);
 
-    for (l = new_buttons; l; l = l->next)
-    {
-        GtkWidget *container;
-        container = BUTTON_DATA (l->data)->container;
-        gtk_box_append (GTK_BOX (self->buttons_box), container);
-    }
-
     /* Buttons are listed in reverse order such that the current location is
      * always the first link. */
     self->button_list = g_list_reverse (new_buttons);
+
+    for (l = self->button_list; l; l = l->next)
+    {
+        GtkWidget *container;
+        container = BUTTON_DATA (l->data)->container;
+        gtk_box_prepend (GTK_BOX (self->buttons_box), container);
+    }
 }
 
 void
