@@ -20,17 +20,20 @@
 
 #include <gtk/gtk.h>
 #include <gio/gio.h>
+#include <libadwaita-1/adwaita.h>
 
 G_BEGIN_DECLS
 
 #define NAUTILUS_TYPE_SPECIAL_LOCATION_BAR (nautilus_special_location_bar_get_type ())
-G_DECLARE_FINAL_TYPE (NautilusSpecialLocationBar, nautilus_special_location_bar, NAUTILUS, SPECIAL_LOCATION_BAR, GtkInfoBar)
+G_DECLARE_FINAL_TYPE (NautilusSpecialLocationBar, nautilus_special_location_bar, NAUTILUS, SPECIAL_LOCATION_BAR, AdwBin)
 
 typedef enum {
-	NAUTILUS_SPECIAL_LOCATION_TEMPLATES,
-	NAUTILUS_SPECIAL_LOCATION_SCRIPTS,
+    NAUTILUS_SPECIAL_LOCATION_TEMPLATES,
+    NAUTILUS_SPECIAL_LOCATION_SCRIPTS,
+    NAUTILUS_SPECIAL_LOCATION_SHARING,
+    NAUTILUS_SPECIAL_LOCATION_TRASH,
 } NautilusSpecialLocation;
 
 GtkWidget	*nautilus_special_location_bar_new (NautilusSpecialLocation location);
 
-G_END_DECLS
\ No newline at end of file
+G_END_DECLS
